class MinimumIdentifierReached(Exception):
    pass


class UninitializedContextIteration(Exception):
    pass


class ContextIterationNotFound(Exception):
    pass


class InvalidSpecialFieldName(Exception):
    pass


class InvalidSpecialFieldValue(Exception):
    pass


class SpecialFieldNotFound(Exception):
    pass


class VarNotFound(Exception):
    pass
